--[[
	Copyright (C) Udorn (Blackhand)
--]]

petbm = petbm or {}
   
petbm.PetTeamView = petbm.PetBattleMaster:NewModule("PetTeamView", "AceEvent-3.0", "AceHook-3.0")
local L = petbm.Locale.GetInstance()

local log = petbm.Debug:new("PetTeamView")

local MAX_ACTIVE_PETS = 3
local MAX_TEAM_FRAMES = 6

local BACKDROP_COLOR = {31/255, 28/255, 38/255, 0}
local BACKDROP_BORDER_COLOR = {1, 1, 1, 1}

local BACKDROP = {
	bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", 
	edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", 
	tile = true, tileSize = 16, edgeSize = 16, 
	insets = { left = 4, right = 4, top = 4, bottom = 4 }
}

local function _SaveCurrentTeam(self)

	if (self.isSelecting) then
		return
	end
	
	log:Debug("_SaveCurrentTeam [%s]", self.currentTeam.name)
	for i=1,MAX_ACTIVE_PETS do
		local petId = C_PetJournal.GetPetLoadOutInfo(i)
		self.currentTeam.pets[i] = petId
		log:Debug("SAVE [%s]", petId)
	end
end

local function _UpdatePortrait(self, portrait)
	local petId, ability1ID, ability2ID, ability3ID, locked = C_PetJournal.GetPetLoadOutInfo(portrait:GetID())
	if (petId <= 0) then
		portrait.icon:Hide()
		portrait.level:SetText("")
	else
		local speciesID, customName, level, xp, maxXp, displayID, name, icon, petType, creatureId = C_PetJournal.GetPetInfoByPetID(petId)
		local health, maxHealth, attack, speed, quality = C_PetJournal.GetPetStats(petId)
		portrait.petTypeIcon:SetTexture("Interface\\PetBattles\\PetIcon-"..PET_TYPE_SUFFIX[petType])
		portrait.icon:SetTexture(icon)
		portrait.level:SetText(level)
		portrait.icon:Show()
		portrait.healthBar:Hide()
		local color = ITEM_QUALITY_COLORS[1]
		if (quality) then
			color = ITEM_QUALITY_COLORS[quality-1]
		end
		portrait.iconBorder:SetVertexColor(color.r, color.g, color.b, 1)
		
		if (health <= 0) then
			portrait.isDead:Show()
		else
			portrait.isDead:Hide()
			if (health < maxHealth) then
				portrait.healthBar:Update(health, maxHealth)
				portrait.healthBar:Show()
			end
		end
	end
end

local function _SelectTeam(self, id)
	log:Debug("SelectTeam [%s]", id)
	
	if (self.isSelecting) then
		return
	end
	
	self.isSelecting = true
	
	self.db.profile.currentTeam = id
	self.currentTeam = self.db.profile.teams[id]
	self.teamName:SetText(self.currentTeam.name)
	
	for i=1,MAX_ACTIVE_PETS do
		local petId = self.currentTeam.pets[i]
		if (petId) then
			log:Debug("LOAD [%s]", petId)
			C_PetJournal.SetPetLoadOutInfo(i, petId)
			if (PetJournal_UpdatePetLoadOut) then
				PetJournal_UpdatePetLoadOut()
			end
		else
			-- remember current selection
			self.currentTeam.pets[i] = C_PetJournal.GetPetLoadOutInfo(i)
		end
	end
	
	self.selectDelay = time() + 1
	self.isSelecting = nil
end

local function _OnTeamSelected(frame)
	_SelectTeam(frame.obj, frame:GetID())
end

local function _GetTeamSelectTooltip(frame)
	local self = frame.obj
	local team = self.db.profile.teams[frame:GetID()]
	if (team) then
		return L["Selects the team \"%s\""]:format(team.name)
	end
	return ""
end

local function _GetTeamEditTooltip(frame)
	local self = frame.obj
	if (self.currentTeam) then
		return L["Edits the name and the icon of team \"%s\""]:format(self.currentTeam.name)
	end
	return ""
end

local function _UpdateTeamFrames(self)
	for i=1,MAX_TEAM_FRAMES do
		local teamFrame = self.teamFrames[i]
		local team = self.db.profile.teams[i]
		teamFrame.icon:SetTexture(team.icon)
	end
end

local function _UpdateTeamFrame(frame)
	local self = frame.obj
	local hidden = self.isSelecting or (self.selectDelay and time() < self.selectDelay)
	if (hidden) then
		frame:Disable()
		frame.icon:SetDesaturated(1)
	else
		frame:Enable()
		frame.icon:SetDesaturated(nil)
		if (self.currentTeam) then
			self.teamName:SetText(self.currentTeam.name)
		end
	end
	if (frame:GetID() == self.db.profile.currentTeam) then
		frame.iconBorder:Show()
	else
		frame.iconBorder:Hide()
	end
end

local function _OnShowPopup(popup)
	local self = popup.obj
	if (not self.currentTeam) then
		popup:Hide()
		return
	end
	popup.editBox:SetText(self.currentTeam.name)
end

local function _OnSavePopup(ok)
	local self = ok:GetParent().obj
	local popup = self.popup
	if (not self.currentTeam) then
		popup:Hide()
		return
	end
	self.currentTeam.name = popup.editBox:GetText() or ""
	popup:Hide()
end

local function _TogglePopup(frame)
	local self = frame.obj
	if (self.popup:IsVisible()) then
		self.popup:Hide()
	else
		self.popup:Show()
	end
end

local function _InitPopup(self)
	self.popup.obj = self
	self.popup.nameLabel:SetText(L["Team name"])
	self.popup:SetScript("OnShow", _OnShowPopup)
	self.popup.ok:SetScript("OnClick", _OnSavePopup)
end
 
local function _InitPortraitFrame(self, parent, id)
	local frame = CreateFrame("Frame", "petbmTeamMember"..id, parent, "petbmTeamMemberTemplate") 
	frame:SetID(id)
	petbm.WidgetUtil.AddTooltip(frame, L["Pet portrait"], L["Enter pet journal to switch between pets"])
	
	local healthBar = petbm.HealthBar.New(parent, frame:GetWidth() - 2, 8)
	healthBar.frame:SetPoint("TOPLEFT", frame, "TOPLEFT", 4, 0)
	frame.healthBar = healthBar
	
	return frame
end

local function _InitTeamFrame(self, parent, id)
	local frame = CreateFrame("Button", "petbmTeam"..id, parent, "petbmTeamTemplate")
	self.teamFrames[id] = frame
	frame.obj = self
	frame:SetID(id)
	frame:SetScript("OnClick", _OnTeamSelected)
	frame:SetScript("OnUpdate", _UpdateTeamFrame)
	
	petbm.WidgetUtil.AddTooltip(frame, L["Team selection"], _GetTeamSelectTooltip)
	
	return frame
end

local function _InitFrame(self)
	local frame = CreateFrame("Frame", "petbmTeamView", UIParent)--, "SecureFrameTemplate")
	frame.obj = self
	frame:SetWidth(270)
	frame:SetHeight(140)
	frame:SetPoint("TOP", UIParent, "TOP", 0, -30)
	frame:SetScript("OnUpdate", function(frame) frame.obj:Update() end)
	frame:SetMovable(true)
	frame:Hide()
	self.frame = frame
	petbm.MovingView.RegisterFrame(frame)
	
	frame:SetBackdrop(BACKDROP)
	frame:SetBackdropColor(unpack(BACKDROP_COLOR))
	
	local background = frame:CreateTexture(nil, "BACKGROUND")
	self.background = background
	background:SetPoint("TOPLEFT", frame, "TOPLEFT", 4, -4)
	background:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", -4, 4)
	--background:SetTexture("Interface\\PetBattles\\MountJournal-BG")
	background:SetTexture("Interface\\PetBattles\\PetJournal")
	background:SetTexCoord(0.00195313, 0.79882813, 0.59277344, 0.75976563)
	--background:SetTexCoord(0.00195313, 0.79101563, 0.48730469, 0.59082031)

	-- pet menu
	local petJournal = CreateFrame("Button", nil, frame, "MainMenuBarMicroButton")
	petJournal:SetPoint("TOPLEFT", frame, "TOPLEFT", 15, -24)
	LoadMicroButtonTextures(petJournal, "Mounts")
	petJournal.tooltipText = MicroButtonTooltipText(MOUNTS_AND_PETS, "TOGGLEPETJOURNAL")
	petJournal.newbieText = NEWBIE_TOOLTIP_MOUNTS_AND_PETS
	petJournal:SetScript("OnClick", function() TogglePetJournal(2) end)	
	
	-- team portraits
	local portrait1 = _InitPortraitFrame(self, frame, 1)
	portrait1:SetPoint("LEFT", petJournal, "RIGHT", 10, 0)
	self.portrait1 = portrait1
	
	local portrait2 = _InitPortraitFrame(self, frame, 2)
	portrait2:SetPoint("LEFT", portrait1, "RIGHT", 20, 0)
	self.portrait2 = portrait2

	local portrait3 = _InitPortraitFrame(self, frame, 3)
	portrait3:SetPoint("LEFT", portrait2, "RIGHT", 20, 0)
	self.portrait3 = portrait3

	-- change team name and icon
	local change = CreateFrame("Button", nil, frame)
	change.obj = self
	change:SetWidth(16)
	change:SetHeight(16)
	local texture = change:CreateTexture()
	texture:SetTexture("Interface\\Addons\\PetBattleMaster\\resources\\Edit-Up")
	texture:SetAllPoints(change)
	texture:SetTexCoord(0, 19/32, 0, 18/32)
	change:SetNormalTexture(texture)
	local texture = change:CreateTexture()
	texture:SetTexture("Interface\\Addons\\PetBattleMaster\\resources\\Edit-Down")
	texture:SetAllPoints(change)
	texture:SetTexCoord(0, 19/32, 0, 18/32)
	change:SetPushedTexture(texture)
	change:SetPoint("TOPLEFT", frame, "TOPLEFT", 12, -12)
	change:SetScript("OnClick", _TogglePopup)
	petbm.WidgetUtil.AddTooltip(change, L["Team name/icon edit"], _GetTeamEditTooltip)

	-- team name
	local teamName = frame:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	teamName:SetJustifyH("LEFT")
	teamName:SetWidth(200)
	teamName:SetPoint("LEFT", change, "RIGHT", 10, 0)
--	teamName:SetPoint("TOPLEFT", frame, "TOPLEFT", 12, -12)
	teamName:SetText("Team 1")
	self.teamName = teamName
	
	-- close button
	local close = CreateFrame("Button", nil, frame, "UIPanelCloseButton")
	close:SetPoint("TOPRIGHT", frame, "TOPRIGHT", 0, 0)
	close:SetScript("OnClick", function(frame) frame:GetParent():Hide() end)
	
	-- team buttons
	for i=1,MAX_TEAM_FRAMES do
		local teamButton = _InitTeamFrame(self, frame, i)
		if (i == 1) then
			teamButton:SetPoint("BOTTOMLEFT", frame, "BOTTOMLEFT", 12, 7)
		else
			teamButton:SetPoint("LEFT", self.teamButtons[i - 1], "RIGHT", 2, 0)
		end
		self.teamButtons[i] = teamButton
	end
	
	-- heal button
	-- TODO the parent has also to be secure, but that's not enough...
--	local heal = CreateFrame("Button", nil, frame, "PetbmHealPetButtonTemplate")
--	heal:SetPoint("TOPRIGHT", frame, "TOPRIGHT", -15, -30)
--	self.heal = heal
		
end

local function ProfileUpdate(self)
	_UpdateTeamFrames(self)
end

local function SetPetLoadOutInfo(self, id, petId)
	log:Debug("SetPetLoadOutInfo id [%s] petId [%s]", id, petId)
	local oldPetId = self.currentTeam.pets[id]
	for i=1,MAX_ACTIVE_PETS do
		if (self.currentTeam.pets[i] == petId and i ~= id) then
			self.currentTeam.pets[i] = oldPetId
		end
	end 
	self.currentTeam.pets[id] = petId
end

local function PET_BATTLE_CLOSE(self)
	self.frame:Show()
end

local function PET_BATTLE_OPENING_START(self)
	self.frame:Hide()
end

local function Update(self)
--	if (self.heal and PetJournalHealPetButton_OnLoad) then
--		PetJournalHealPetButton_OnLoad(self.heal)
--		self.heal = nil
--	end
	
--	if (not self.hooked and PetJournalDragButton_OnDragStart) then
--		log:Debug("HOOK")
--		self:SecureHook("PetJournalDragButton_OnDragStart", function() log:Debug("DRAG") end)
--		self.hooked = true
--	end
	self.background:SetAlpha(petbm.PetBattleMaster.db.profile.teamViewAlpha)
	self.frame:SetBackdropBorderColor(unpack(petbm.PetBattleMaster.db.profile.teamViewBorderColor))
	_UpdatePortrait(self, self.portrait1)
	_UpdatePortrait(self, self.portrait2)
	_UpdatePortrait(self, self.portrait3)
end

local function OnInitialize(self)
	self.teamButtons = {}
	self.teamFrames = {}
	self.db = petbm.PetBattleMaster.db:RegisterNamespace("PetTeamView", {
		profile = {
			currentTeam = 1,
    		teams = {
    			{
    				name = L["Team %s"]:format(1),
    				icon = "Interface\\Icons\\ability_deathknight_summongargoyle",
    				pets = {}
    			},
    			{
    				name = L["Team %s"]:format(2),
    				icon = "Interface\\Icons\\Ability_Druid_FerociousBite",
    				pets = {}
    			},
    			{
    				name = L["Team %s"]:format(3),
    				icon = "Interface\\Icons\\Ability_EyeOfTheOwl",
    				pets = {}
    			},
    			{
    				name = L["Team %s"]:format(4),
    				icon = "Interface\\Icons\\Ability_Hunter_Pet_Devilsaur",
    				pets = {}
    			},
    			{
    				name = L["Team %s"]:format(5),
    				icon = "Interface\\Icons\\achievment_Boss_ultraxion",
    				pets = {}
    			},
    			{
    				name = L["Team %s"]:format(6),
    				icon = "Interface\\Icons\\INV_Pet_Cats_OrangeTabbyCat",
    				pets = {}
    			}
    		}
    	}
    })
    self.currentTeam = self.db.profile.teams[self.db.profile.currentTeam]
	_InitFrame(self)
end

local function OnEnable(self)
	self:RegisterEvent("PET_BATTLE_OPENING_START")
	self:RegisterEvent("PET_BATTLE_CLOSE")
	-- sometimes we don't get the event, so we take a hook
	self:SecureHook(C_PetJournal, "SetPetLoadOutInfo")
	
	self.popup = _G["petbmNameIconEditor"]
	_InitPopup(self)
	self:ProfileUpdate()
end

local function ToggleView(self)
	if (self.frame:IsVisible()) then
		self.frame:Hide()
	else
		self.frame:Show()
		_SelectTeam(self, self.db.profile.currentTeam)
		--_SelectTeam(self, 1)
	end	
end

petbm.PetTeamView.OnInitialize = OnInitialize
petbm.PetTeamView.OnEnable = OnEnable
petbm.PetTeamView.ProfileUpdate = ProfileUpdate
petbm.PetTeamView.Update = Update
petbm.PetTeamView.ToggleView = ToggleView
petbm.PetTeamView.PET_BATTLE_CLOSE = PET_BATTLE_CLOSE
petbm.PetTeamView.PET_BATTLE_OPENING_START = PET_BATTLE_OPENING_START
petbm.PetTeamView.SetPetLoadOutInfo = SetPetLoadOutInfo